% Prolog equivalent of YM's sentences
%
% Revisions
%
% 2014-04-18 : YM : added y/m prefix to each (exported) predicate
% 2014-04-15 : YMA :	reintroduced the module definition (no change)
% 2014-04-12 : YMA :	various major revisions
% 2014-04-08 : CMSMcQ : supply uniquely identifying predicate for T
%                       extend with style rules
%                       remove general rules to y_general.pl
% 2014-04-03 : CMSMcQ : initial translation from table_trial_YM.txt

:- module(y, [y_this_table/1,
	      y_table_dimensions/3,
	      y_cell/3,
	      y_table_consistent/1,
	      y_tableWidth/2,
	      y_tableMarginLeft/2,
          y_isTableHeader/3,
          y_tableCellContent/4,
	      y_tableCellBackgroundColor/4,
	      y_tableCellVerticalAlign/4,
	      y_tableCellBorderWidth/4,
	      y_tableCellBorderStyle/4
	     ]).

% Individuals: for each individual that needs one, we have a uniquely
% identifying predicate. (Here, we have only one individual needing
% such a predicate:  the table.)
% t is a table; it is the only individual we identify
% apart from the natural numbers and the strings

y_this_table(t).

y_tableWidth(t,"80%").
y_tableMarginLeft(t,"10%").

y_isTableHeader(t, 1, 1).
y_isTableHeader(t, 1, 2).

y_tableCellContent(t, 1, 1, "Anne").
y_tableCellContent(t, 1, 2, "vnement").

y_tableCellContent(t, 2, 1, "1969").
y_tableCellContent(t, 3, 1, "1992").

y_tableCellContent(t, 2, 2, "Cration d'ARPANET, le premier rseau national amricain d'ordinateurs, par le Defense Department's Advanced Research Projects Agency (DARPA)").
y_tableCellContent(t, 3, 2, "Mise en service du World Wide Web par le CERN (Centre europen de recherche nuclaire), en Suisse").

% Uncomment to test for incorrectness
% y_tableCellContent(t, 4, 4, "Extraneous").

% The style is consistent across cells in the table, so we can
% have some general rules.

y_cell(T,R,C) :- y_tableCellContent(T, R, C, _).

y_tableCellBackgroundColor(t,R,C,"#CCCCCC") :- y_isTableHeader(t,R,C).
y_tableCellVerticalAlign(t,R,C,"top") :- y_cell(t,R,C).
y_tableCellBorderWidth(t,R,C,"thin") :- y_cell(t,R,C).
y_tableCellBorderStyle(t,R,C,"solid") :- y_cell(t,R,C).

% General rules about tables. We formulate some of these as tests of
% consistent description for a table.

table_nonempty_finite_rectangular_and_dense(T) :-
	y_cell(T, R, C),
	\+ (
		y_cell(T, Row, Col),
		(Row > R;
		Col > C)
	   ;
		between(1, R, Row),
		between(1, C, Col),
		(\+ y_cell(T, Row, Col))
	   ).

table_properties_ok(T) :-
	\+ (has_properties(T,Row,Col),
	      \+ cell(T,Row,Col)).

has_properties(T, R, C) :-
	y_isTableHeader(T,R,C);
	y_tableCellBackgroundColor(T,R,C,_);
	y_tableCellVerticalAlign(T,R,C,_);
	y_tableCellBorderWidth(T,R,C,_);
	y_tableCellBorderStyle(T,R,C,_).

y_table_consistent(T) :-
	table_nonempty_finite_rectangular_and_dense(T),
	table_properties_ok(T).

y_table_dimensions(T,Row,Col) :-
% This is not a validity check, but rather a simple way
% to compute a table's dimensions.
% Note: Reliable only if table_nonempty_finite_rectangular_and_dense(T)
	y_cell(T,Row,Col),
	R is Row + 1,
	C is Col + 1,
	\+ y_cell(T, R, 1),
	\+ y_cell(T, 1, C).
