% Inferences from the XHTML table

% 2014-04-18 : YM : added y/m prefix to each (exported) predicate
% 2014-04-17 : YM :  Corrected definitions of row(R) and col(C)
                     (R and C were at a wrong place in the predicate
                     on the right)
% 2014-04-08 : MSM : add uniquely identifying predicates for all
%                    individuals.
% 2014-03-21 : MSM : transcribed ---28.

:- module(m, [m_headingcell/1,
	      m_datacell/1,
	      m_cell/1,
	      m_row/1,
	      m_column/1,
	      m_this_table/1,
	      m_table_row/3,
	      m_table_column/3,
	      m_table_cell/4,
	      m_table_rowsequence/2,
	      m_table_colsequence/2,
	      m_row_cellsequence/2,
	      m_col_cellsequence/2,
	      m_cell_text/2,
	      m_table_row/2,
	      m_table_col/2,
	      m_table_cell/2,
	      m_row_cell/2,
	      m_col_cell/2]).

% Individuals
m_this_table(t1).
m_table_row(r1, t1, 1).
m_table_row(r2, t1, 2).
m_table_row(r3, t1, 3).
m_table_column(c1, t1, 1).
m_table_column(c2, t1, 2).

% convention:  cell X, Y is cXY
m_table_cell(c11,t1,1,1).
m_table_cell(c21,t1,1,2).
m_table_cell(c12,t1,2,1).
m_table_cell(c22,t1,2,2).
m_table_cell(c13,t1,3,1).
m_table_cell(c23,t1,3,2).

% basic classes
m_row(R) :- m_table_row(R,_T,_N).
m_column(C) :- m_table_column(C,_T,_N).
m_headingcell(c11).
m_headingcell(c21).
m_datacell(c12).
m_datacell(c22).
m_datacell(c13).
m_datacell(c23).

% Relations
%

m_table_rowsequence(t1, [r1, r2, r3]).
m_table_colsequence(t1, [c1, c2]).

m_row_cellsequence(r1, [c11, c21]).
m_row_cellsequence(r2, [c12, c22]).
m_row_cellsequence(r3, [c13, c23]).

m_col_cellsequence(c1, [c11, c12, c13]).
m_col_cellsequence(c2, [c21, c22, c23]).

m_cell_text(c11, "Anne").
m_cell_text(c21, "vnement").
m_cell_text(c12, "1969").
m_cell_text(c22, "Cration d'ARPANET, le premier rseau national amricain d'ordinateurs, par le Defense Department's Advanced Research Projects Agency (DARPA)").
m_cell_text(c13, "1992").
m_cell_text(c23, "Mise en service du World Wide Web par le CERN (Centre europen de recherche nuclaire), en Suisse").


% Synonyms / analytic sentences
% These are not essential, but may be convenient to have.

m_cell(X) :- m_headingcell(X).
m_cell(X) :- m_datacell(X).

m_table_row(T,R) :- m_this_table(T), m_row(R), m_table_rowsequence(T,Rs), member(R,Rs).
m_table_col(T,C) :- m_this_table(T), m_column(C), m_table_colsequence(T,Cs), member(C,Cs).
m_row_cell(R,C) :- m_row(R), m_cell(C), m_row_cellsequence(R,Cs), member(C,Cs).
m_col_cell(Col,Cell) :- m_column(Col),
	m_cell(Cell),
	m_col_cellsequence(Col, Cells),
	member(Cell,Cells).

m_table_cell(T,C) :- 
	m_this_table(T),
	m_cell(C),
	m_row(R),
	m_table_row(T,R),
	m_row_cell(R,C).

m_table_cell(T,C) :-
	m_this_table(T),
	m_cell(C),
	m_column(C2),
	m_table_col(T,C2),
	m_col_cell(C2,C).
